///
//  PDFFilterView.m
//  PDFReader
//
//  Copyright © 2014-2022 PDF Technologies, Inc. All Rights Reserved.
//
//  The PDF Reader Sample applications are licensed with a modified BSD license.
//  Please see License for details. This notice may not be removed from this file.
//

#import "PDFFilterView.h"

@interface PDFFilterView () <UITableViewDelegate, UITableViewDataSource>

@property (nonatomic,retain) UIView *backgroundView;

@property (nonatomic,retain) UITableView *tableView;

@end


@implementation PDFFilterView

#pragma mark - Init Methods

- (instancetype)init {
    if (self = [super init]) {
        self.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight;
        self.clipsToBounds = YES;
        
        _backgroundView = [[UIView alloc] init];
        _backgroundView.autoresizingMask = UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight;
        _backgroundView.backgroundColor = [UIColor blackColor];
        [self addSubview:_backgroundView];
        
        _tableView = [[UITableView alloc] init];
        _tableView.autoresizingMask = UIViewAutoresizingFlexibleWidth;
        _tableView.dataSource = self;
        _tableView.delegate = self;
        [self addSubview:_tableView];
    }
    return self;
}

- (void)dealloc {
    Block_release(_didSelected);
    [_selectedIndexes release];
    [_items release];
    _tableView.delegate = nil;
    _tableView.dataSource = nil;
    [_tableView release];
    [_backgroundView release];
    [super dealloc];
}

#pragma mark - Public Methods

- (void)showInView:(UIView *)view {
    [view addSubview:self];
    
    self.backgroundView.frame = self.bounds;
    self.backgroundView.alpha = 0.0;
    self.tableView.frame = CGRectMake(0, -132, self.frame.size.width, 132);
    self.userInteractionEnabled = NO;
    [UIView animateWithDuration:0.3 animations:^{
        self.backgroundView.alpha = 0.5;
        self.tableView.frame = CGRectMake(0, 0, self.frame.size.width, 132);
    } completion:^(BOOL finished) {
        self.userInteractionEnabled = YES;
    }];
    
    [self.tableView reloadData];
}

- (void)dismiss {
    self.userInteractionEnabled = NO;
    [UIView animateWithDuration:0.3 animations:^{
        self.backgroundView.alpha = 0.0;
        self.tableView.frame = CGRectMake(0, -132, self.frame.size.width, 132);
    } completion:^(BOOL finished) {
        [self removeFromSuperview];
        self.userInteractionEnabled = YES;
        self.didSelected = nil;
    }];
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event {
    [super touchesBegan:touches withEvent:event];
    UITouch *touch = touches.anyObject;
    CGPoint point = [touch locationInView:self];
    if (!CGRectContainsPoint(self.tableView.frame, point)) {
        self.userInteractionEnabled = NO;
        [UIView animateWithDuration:0.3 animations:^{
            self.backgroundView.alpha = 0.0;
            self.tableView.frame = CGRectMake(0, -132, self.frame.size.width, 132);
        } completion:^(BOOL finished) {
            [self removeFromSuperview];
            self.userInteractionEnabled = YES;
            if (self.selectedIndexes) {
                self.didSelected(-1);
                self.didSelected = nil;
            }
        }];
    }
}

#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.items count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"cell"];
    if (!cell) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"cell"] autorelease];
        cell.textLabel.textColor = [UIColor colorWithRed:74.0/255.0 green:74.0/255.0 blue:74.0/255.0 alpha:1.0] ;
        cell.textLabel.font = [UIFont systemFontOfSize:15];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
    }
    id item = self.items[indexPath.row];
    if ([item isKindOfClass:[UIImage class]]) {
        cell.imageView.image = item;
    } else if ([item isKindOfClass:[NSString class]]) {
        cell.textLabel.text = item;
    }
    if ([self.selectedIndexes containsObject:@(indexPath.row)]) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    } else {
        cell.accessoryType = UITableViewCellAccessoryNone;
    }
    return cell;
}

#pragma mark - UITableViewDelegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 44;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.allowsMultipleSelection &&
        self.selectedIndexes) {
        NSNumber *selectedNumber = nil;
        for (NSNumber *number in self.selectedIndexes) {
            if ([number integerValue] == indexPath.row) {
                selectedNumber = number;
                break;
            }
        }
        NSMutableArray *array = [NSMutableArray arrayWithArray:self.selectedIndexes];
        if (selectedNumber) {
            [array removeObject:selectedNumber];
        } else {
            [array addObject:@(indexPath.row)];
        }
        self.selectedIndexes = array;
    } else {
        self.selectedIndexes = @[@(indexPath.row)];
    }
    [self.tableView reloadData];
    if (self.didSelected) {
        self.didSelected(indexPath.row);
    }
}

@end
