//
//  PDFKeyboardToolbar.h
//  PDFReader
//
//  Copyright © 2014-2022 PDF Technologies, Inc. All Rights Reserved.
//
//  The PDF Reader Sample applications are licensed with a modified BSD license.
//  Please see License for details. This notice may not be removed from this file.
//

#import <UIKit/UIKit.h>
#import "PDFKTSlideView.h"
#import "PDFKTColorPickView.h"
#import "PDFKTFontNameView.h"
#import "PDFSettingFontView.h"

@class PDFKeyboardToolbar;
@protocol PDFKeyboardDelegate <NSObject>

@optional
- (void)keyboardShouldDissmiss:(PDFKeyboardToolbar *)toolbar;
- (void)keyboard:(PDFKeyboardToolbar *)toolbar updateFontName:(NSString *)fontName;
- (void)keyboard:(PDFKeyboardToolbar *)toolbar updateFontSize:(CGFloat)fontSize;
- (void)keyboard:(PDFKeyboardToolbar *)toolbar updateTextColor:(UIColor *)textColor;
- (void)keyboard:(PDFKeyboardToolbar *)toolbar updateOpacity:(CGFloat)opacity;
@end

@interface PDFKeyboardToolbar : UIView <UITextViewDelegate, UITextFieldDelegate,PDFKTSlideDelegate,PDFKTColorPickDelegate,PDFKTfontNamesDelegate,PDFKTSizeDelegate>

@property (nonatomic,copy) NSString *fontName;
@property (nonatomic,assign) CGFloat fontSize;
@property (nonatomic,assign) CGFloat opacity;
@property (nonatomic, assign) id<PDFKeyboardDelegate> delegate;

- (id)initWithViews:(NSArray*)buttons;
- (id)initWithFontName:(NSString *)fontName fontSize:(CGFloat)fontSize;
- (void)bindToTextView:(UITextView *)textView;
- (void)bindToTextField:(UITextField *)textField;
- (void)refreshFontName:(NSString *)fontName fontSize:(CGFloat)fontSize opacity:(CGFloat)opacity;
@end

