//
//  StampTextCell.h
//  PDFReader
//
//  Copyright © 2014-2022 PDF Technologies, Inc. All Rights Reserved.
//
//  The PDF Reader Sample applications are licensed with a modified BSD license.
//  Please see License for details. This notice may not be removed from this file.
//

#import <UIKit/UIKit.h>
#import "StampPreview.h"

typedef enum {
    CustomTextCellType_Text,
    CustomTextCellType_Color,
    CustomTextCellType_Date,
    CustomTextCellType_Time,
    CustomTextCellType_Preview
}CustomTextCellType;


@protocol StampTextCellDelegate <NSObject>

@required
- (void)switchValueChanged_Date:(id)sender;

- (void)switchValueChanged_Time:(id)sender;

- (void)textFieldWithText_Text:(id)sender;

- (void)buttonItemClicked_Color:(id)sender;

@end



@interface StampTextCell : UITableViewCell <UITextFieldDelegate> {
    id <StampTextCellDelegate> _delegate;
    CustomTextCellType           _cellType;
    
    UIButton *btnBlack;
    UIButton *btnGreen;
    UIButton *btnRed;
    UIButton *btnBlue;
    
    UIButton *leftGreen;
    UIButton *leftRed;
    UIButton *leftBlue;
    
    UIButton *rightGreen;
    UIButton *rightRed;
    UIButton *rightBlue;
    
    StampPreview    *_preView;
}

@property (nonatomic,assign) id delegate;
@property (nonatomic,assign) CustomTextCellType cellType;
@property (nonatomic,retain) StampPreview *preView;
@property (nonatomic,retain) UITextField *stampTextField;
@property (nonatomic,retain) UISwitch *haveDateSwitch;
@property (nonatomic,retain) UISwitch *haveTimeSwitch;

- (void)setCellStyle:(CustomTextCellType)cellType label:(NSString *)label;

- (void)preViewReDisplay;

- (void)selectColor:(NSInteger)buttonColor;


@end
