//
//  PDFListView.h
//  PDFReader
//
//  Copyright © 2014-2022 PDF Technologies, Inc. All Rights Reserved.
//
//  The PDF Reader Sample applications are licensed with a modified BSD license.
//  Please see License for details. This notice may not be removed from this file.
//

#import <ComPDFKit/ComPDFKit.h>

typedef NS_ENUM(NSInteger, PDFViewAnnotationMode) {
    PDFViewAnnotationModeNone = 0,
    PDFViewAnnotationModeNote,
    PDFViewAnnotationModeHighlight,
    PDFViewAnnotationModeUnderline,
    PDFViewAnnotationModeStrikeout,
    PDFViewAnnotationModeSquiggly,
    PDFViewAnnotationModeCircle,
    PDFViewAnnotationModeSquare,
    PDFViewAnnotationModeArrow,
    PDFViewAnnotationModeLine,
    PDFViewAnnotationModeInk,
    PDFViewAnnotationModeFreeText,
    PDFViewAnnotationModeStamp,
    PDFViewAnnotationModeImage,
    PDFViewAnnotationModeLink,
};

@class PDFListView;

@protocol PDFListViewDelegate <NSObject>

@optional

- (void)PDFViewPerformOpenNote:(PDFListView *)pdfView forAnnotation:(CPDFAnnotation *)annotation;

- (void)PDFViewPerformChangeColor:(PDFListView *)pdfView forAnnotation:(CPDFAnnotation *)annotation;

- (void)PDFViewPerformShare:(PDFListView *)pdfView forAnnotation:(CPDFMarkupAnnotation *)annotation;

- (void)PDFViewPerformSave:(PDFListView *)pdfView forAnnotation:(CPDFStampAnnotation *)annotation;

- (void)PDFViewPerformPopup:(PDFListView *)pdfView forAnnotation:(CPDFMarkupAnnotation *)annotation;

- (void)PDFViewPerformEditLink:(PDFListView *)pdfView forAnnotation:(CPDFLinkAnnotation *)annotation;

- (void)PDFViewPerformSignatureWidget:(PDFListView *)pdfView forAnnotation:(CPDFSignatureWidgetAnnotation *)annotation;

- (void)PDFViewPerformShare:(PDFListView *)pdfView forSelection:(CPDFSelection *)selection;

- (void)PDFViewPerformDefine:(PDFListView *)pdfView forSelection:(CPDFSelection *)selection;

- (void)PDFViewPerformGoogleSearch:(PDFListView *)pdfView forSelection:(CPDFSelection *)selection;

- (void)PDFViewPerformWikiSearch:(PDFListView *)pdfView forSelection:(CPDFSelection *)selection;

- (void)PDFViewPerformAddSignture:(PDFListView *)pdfView atPoint:(CGPoint)point forPage:(CPDFPage *)page;

- (void)PDFViewPerformAddStamp:(PDFListView *)pdfView atPoint:(CGPoint)point forPage:(CPDFPage *)page;

- (void)PDFViewPerformAddImage:(PDFListView *)pdfView atPoint:(CGPoint)point forPage:(CPDFPage *)page;

- (void)PDFViewPerformTouchEnded:(PDFListView *)pdfView;

- (void)PDFViewPerformWillGoTo:(PDFListView *)pdfView pageIndex:(NSInteger)pageIndex;

@end

@interface PDFListView : CPDFView

@property (nonatomic,assign) id<PDFListViewDelegate> performDelegate;

@property (nonatomic,assign) PDFViewAnnotationMode annotationMode;

@property (nonatomic,retain) CPDFAnnotation *activeAnnotation;

@property (nonatomic,retain) CPDFAnnotation *addAnnotation;

@property (nonatomic,readonly) CGPoint menuPoint;
@property (nonatomic,readonly) CPDFPage *menuPage;

- (void)addAnnotation:(CPDFAnnotation *)annotation forPage:(CPDFPage *)page;

- (void)addAnnotation:(CPDFAnnotation *)annotation;

@end
