//
//  ColorView.h
//  PDFReader
//
//  Copyright © 2014-2022 PDF Technologies, Inc. All Rights Reserved.
//
//  The PDF Reader Sample applications are licensed with a modified BSD license.
//  Please see License for details. This notice may not be removed from this file.
//

#import "ColorView.h"

@interface ColorView ()

@property (nonatomic,retain) NSArray *buttons;

@property (nonatomic,retain) UIView *selectedView;

@property (nonatomic,retain) UIScrollView *scrollView;

@end

@implementation ColorView

#pragma mark Init Methods

- (void)awakeFromNib {
    [super awakeFromNib];
    
    UIScrollView *scrollView = [[[UIScrollView alloc] initWithFrame:self.bounds] autorelease];
    scrollView.showsVerticalScrollIndicator = NO;
    scrollView.showsHorizontalScrollIndicator = NO;
    [self addSubview:scrollView];
    self.scrollView = scrollView;
    
    _selectedView = [[UIView alloc] init];
    _selectedView.backgroundColor = [UIColor clearColor];
    _selectedView.layer.masksToBounds = YES;
    _selectedView.layer.borderColor = [UIColor lightGrayColor].CGColor;
    _selectedView.layer.borderWidth = 1.0;
    [scrollView addSubview:_selectedView];
    
    NSArray *colors = @[[UIColor colorWithRed:221.0f/255.0f green:2.0f/255.0f blue:2.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:1.0f green:27.0f/255.0f blue:137.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:254.0f/255.0f green:168.0f/255.0f blue:189.0f/255.0f alpha:1.0f],
                        
                        [UIColor colorWithRed:39.0f/255.0f green:190.0f/255.0f blue:253.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:0.0f/255.0f green:255.0f/255.0f blue:54.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:252.0f/255.0f green:255.0f/255.0f blue:54.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:253.0f/255.0f green:126.0f/255.0f blue:21.0f/255.0f alpha:1.0f],
                        
                        [UIColor colorWithRed:254.0f/255.0f green:134.0f/255.0f blue:106.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:166.0f/255.0f green:29.0f/255.0f blue:251.0f/255.0f alpha:1.0f],
                        
                        [UIColor colorWithRed:0.0f/255.0f green:129.0f/255.0f blue:68.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:19.0f/255.0f green:51.0f/255.0f blue:144.0f/255.0f alpha:1.0f],
                        [UIColor colorWithRed:1.0f green:1.0f blue:1.0f alpha:1.0f],
                        [UIColor colorWithRed:0.0f green:0.0f blue:0.0f alpha:1.0f]];
    NSMutableArray *buttons = [NSMutableArray array];
    for (int i=0; i <[colors count]; i++){
        UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
        button.backgroundColor = colors[i];
        [button addTarget:self action:@selector(buttonItemClicked_Switch:) forControlEvents:UIControlEventTouchUpInside];
        [scrollView addSubview:button];
        [buttons addObject:button];
    }
    self.buttons = buttons;
}

- (void)layoutSubviews {
    NSInteger count = 7;
    CGFloat offsetX = 5;
    CGFloat space = 10;
    CGFloat width = (self.bounds.size.width-2*offsetX-(count-1)*space)/count;
    if (width < 32) {
        width = 40;
        self.scrollView.frame = CGRectMake(0, 0, self.bounds.size.width, width+2*offsetX);
        self.selectedView.frame = CGRectMake(0, 0, width+2*offsetX, width+2*offsetX);
        self.selectedView.layer.cornerRadius = self.selectedView.bounds.size.width/2.0;
        for (int i=0; i <[self.buttons count]; i++){
            UIButton *button = self.buttons[i];
            button.frame = CGRectMake(offsetX, 5, width, width);
            button.layer.masksToBounds = YES;
            button.layer.cornerRadius = width/2.0;
            if (self.color && [self isTheSameColor:self.color anotherColor:button.backgroundColor]) {
                self.selectedView.center = button.center;
            }
            offsetX += width+space;
        }
        self.scrollView.contentSize = CGSizeMake(offsetX, self.scrollView.bounds.size.height);
    } else {
        self.scrollView.frame = CGRectMake(0, 0, self.bounds.size.width, 2*width+4*offsetX);
        self.selectedView.frame = CGRectMake(0, 0, width+2*offsetX, width+2*offsetX);
        self.selectedView.layer.cornerRadius = self.selectedView.bounds.size.width/2.0;
        for (int i=0; i <[self.buttons count]; i++){
            UIButton *button = self.buttons[i];
            button.frame = CGRectMake(offsetX+(width+space)*(i%count),
                                      offsetX+(width+space)*(i/count), width, width);
            button.layer.masksToBounds = YES;
            button.layer.cornerRadius = width/2.0;
            if (self.color && [self isTheSameColor:self.color anotherColor:button.backgroundColor]) {
                self.selectedView.center = button.center;
            }
        }
    }
    [self invalidateIntrinsicContentSize];
}

- (CGSize)intrinsicContentSize {
    return CGSizeMake(self.bounds.size.width, self.scrollView.bounds.size.height);
}

- (void)dealloc {
    Block_release(_callback);
    [_scrollView release];
    [_selectedView release];
    [_buttons release];
    [_color release];
    [super dealloc];
}

#pragma mark Public Methods

- (void)setColor:(UIColor *)color {
    if (_color != color) {
        [_color release];
        _color = [color retain];
    }
    
    for (UIButton *button in self.buttons) {
        if (self.color && [self isTheSameColor:self.color anotherColor:button.backgroundColor]) {
            self.selectedView.center = button.center;
        }
    }
}

- (void)setAllowsClearColor:(BOOL)allowsClearColor {
    _allowsClearColor = allowsClearColor;
    
    UIButton *button = [UIButton buttonWithType:UIButtonTypeCustom];
    button.backgroundColor = [UIColor clearColor];
    [button setImage:[UIImage imageNamed:@"color_clear_icon.png"] forState:UIControlStateNormal];
    [button addTarget:self action:@selector(buttonItemClicked_Switch:) forControlEvents:UIControlEventTouchUpInside];
    [self.scrollView addSubview:button];

    NSMutableArray *newButtons = [NSMutableArray array];
    [newButtons addObject:button];
    [newButtons addObjectsFromArray:self.buttons];
    self.buttons = newButtons;
}

#pragma mark Private Methods

- (BOOL)isTheSameColor:(UIColor *)color1 anotherColor:(UIColor *)color2 {
    const CGFloat* component = CGColorGetComponents(color1.CGColor);
    const CGFloat* component1 = CGColorGetComponents(color2.CGColor);
    if (fabs(component[0]-component1[0])>1e-6) return NO;
    if (fabs(component[1]-component1[1])>1e-6) return NO;
    if (fabs(component[2]-component1[2])>1e-6) return NO;
    if (fabs(component[3]-component1[3])>1e-6) return NO;
    return YES;
}

#pragma mark Button Actions

- (void)buttonItemClicked_Switch:(UIButton *)button {
    self.selectedView.center = button.center;
    self.color = button.backgroundColor;
    if (self.callback) {
        self.callback(self.color);
    }
}

@end
