//
//  PDFColorViewController.m
//  PDFReader
//
//  Copyright © 2014-2022 PDF Technologies, Inc. All Rights Reserved.
//
//  The PDF Reader Sample applications are licensed with a modified BSD license.
//  Please see License for details. This notice may not be removed from this file.
//

#import "PDFColorViewController.h"
#import "ColorView.h"

@interface PDFColorViewController () <UIPopoverPresentationControllerDelegate>

@property (nonatomic,assign) IBOutlet UIView *contentView;
@property (nonatomic,assign) IBOutlet ColorView *colorView;
@property (nonatomic,assign) IBOutlet UISlider *slider;

@end

@implementation PDFColorViewController

#pragma mark - Init Methods

- (void)dealloc {
    Block_release(_callback);
    [_color release];
    [super dealloc];
}

#pragma mark - UIViewController Methods

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Do any additional setup after loading the view from its nib.
    __block __typeof(self) blockSelf = self;
    self.colorView.callback = ^(UIColor *color) {
        blockSelf.color = color;
        if (blockSelf.callback) {
            blockSelf.callback();
        }
    };
    self.colorView.color = self.color;
    self.slider.value = self.opacity;
}

- (void)viewWillLayoutSubviews {
    [super viewWillLayoutSubviews];
    
    self.preferredContentSize = CGSizeMake(300, self.contentView.bounds.size.height);
}

#pragma mark - Button Actions

- (IBAction)sliderValueChanged_Opacity:(UISlider *)slider {
    self.opacity = slider.value;
    if (self.callback) {
        self.callback();
    }
}

#pragma mark - Public Methods

- (void)showViewController:(UIViewController *)viewController inRect:(CGRect)rect {
    self.modalPresentationStyle = UIModalPresentationPopover;
    UIPopoverPresentationController *popVC = self.popoverPresentationController;
    popVC.delegate = self;
    popVC.sourceRect = rect;
    popVC.sourceView = viewController.view;
    [viewController presentViewController:self animated:YES completion:nil];
}

#pragma mark - UIPopoverPresentationControllerDelegate

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller traitCollection:(UITraitCollection *)traitCollection {
    return UIModalPresentationNone;
}

@end
